from pathlib import Path

import numpy as np

from oct_converter.image_types import OCTVolumeWithMetaData


class POCT(object):
    """Class for extracting data from Optovues's .oct file format.

    Attributes:
        filepath (str): Path to .oct file for reading. Reader expects a file with the same name and a .txt extension
            exists at the same location.
    """

    def __init__(self, filepath):
        self.filepath = Path(filepath)
        if not self.filepath.exists():
            raise FileNotFoundError(self.filepath)
        self.filespec = self.filepath.with_suffix(".txt")
        if not self.filespec.exists():
            raise FileNotFoundError(
                f"Could not find filespec {self.filespec} in the same location as {self.filepath}"
            )

    def _read_filespec(self):
        scan_info = []
        with open(self.filespec, "r", encoding="iso-8859-1") as f:
            lines = f.readlines()
            for i, line in enumerate(lines):
                if "Window Height" in line:
                    height = [int(s) for s in line.split() if s.isdigit()][0]
                if "Scan Length" in line and "Scan Usage" in lines[i + 1]:
                    scan_length = [int(s) for s in line.split() if s.isdigit()][0]
                    scan_number = [int(s) for s in lines[i + 1].split() if s.isdigit()][
                        0
                    ]
                    scan_info.append(
                        {"height": height, "length": scan_length, "number": scan_number}
                    )
        self.scan_info = scan_info

    def read_oct_volume(self):
        """Reads OCT data.
        Args:
            Returns:
                obj:OCTVolumeWithMetaData
        """
        self._read_filespec()

        with open(self.filepath, "rb") as f:
            data = np.frombuffer(
                f.read(), dtype=np.float32
            )  # np.fromstring() gives numpy depreciation warning
            all_volumes = []
            for volume in self.scan_info:
                num_pixels_slice = volume["height"] * volume["length"]
                num_slices = volume["number"]
                all_slices = []
                for i in range(num_slices):
                    slice = data[i * num_pixels_slice : (i + 1) * num_pixels_slice]
                    slice = np.rot90(slice.reshape(volume["length"], volume["height"]))
                    all_slices.append(slice)
                all_volumes.append(OCTVolumeWithMetaData(all_slices))
        return all_volumes



from oct_converter.readers import IMG

filepath = "../sample_files/file.img"
img = IMG(filepath)
oct_volume = (
    img.read_oct_volume()
)  # returns an OCT volume with additional metadata if available
oct_volume.peek()  # plots a montage of the volume
oct_volume.save("img_testing.avi")  # save volume



from oct_converter.readers import POCT

filepath = "../sample_files/sample.OCT"
poct = POCT(filepath)
oct_volumes = poct.read_oct_volume()

for volume in oct_volumes:
    volume.peek()  # plots a montage of the volume
print("debug")



from oct_converter.readers import E2E

filepath = "../sample_files/sample.E2E"
file = E2E(filepath)
oct_volumes = (
    file.read_oct_volume()
)  # returns a list of all OCT volumes with additional metadata if available
for volume in oct_volumes:
    volume.peek(show_contours=True)  # plots a montage of the volume
    volume.save("{}_{}.avi".format(volume.volume_id, volume.laterality))

fundus_images = (
    file.read_fundus_image()
)  # returns a list of all fundus images with additional metadata if available
for image in fundus_images:
    image.save("{}+{}.png".format(image.image_id, image.laterality))



from pathlib import Path

import numpy as np
from construct import ListContainer

from oct_converter.image_types import FundusImageWithMetaData, OCTVolumeWithMetaData
from oct_converter.readers.binary_structs import fds_binary


class FDS(object):
    """Class for extracting data from Topcon's .fds file format.

    Notes:
        Mostly based on description of .fds file format here:
        https://bitbucket.org/uocte/uocte/wiki/Topcon%20File%20Format

    Attributes:
        filepath (str): Path to .img file for reading.
        chunk_dict (dict): Name of data chunks present in the file, and their start locations.
    """

    def __init__(self, filepath):
        self.filepath = Path(filepath)
        if not self.filepath.exists():
            raise FileNotFoundError(self.filepath)

        self.chunk_dict = self.get_list_of_file_chunks()

    def get_list_of_file_chunks(self):
        """Find all data chunks present in the file.

        Returns:
            dict
        """
        chunk_dict = {}
        with open(self.filepath, "rb") as f:
            # skip header
            raw = f.read(15)
            header = fds_binary.header.parse(raw)

            eof = False
            while not eof:
                chunk_name_size = np.fromstring(f.read(1), dtype=np.uint8)[0]
                if chunk_name_size == 0:
                    eof = True
                else:
                    chunk_name = f.read(chunk_name_size)
                    chunk_size = np.fromstring(f.read(4), dtype=np.uint32)[0]
                    chunk_location = f.tell()
                    f.seek(chunk_size, 1)
                    chunk_dict[chunk_name] = [chunk_location, chunk_size]
        print("File {} contains the following chunks:".format(self.filepath))
        for key in chunk_dict.keys():
            print(key)
        return chunk_dict

    def read_oct_volume(self):
        """Reads OCT data.

        Returns:
            obj:OCTVolumeWithMetaData
        """
        if b"@IMG_SCAN_03" not in self.chunk_dict:
            raise ValueError("Could not find OCT header @IMG_SCAN_03 in chunk list")
        with open(self.filepath, "rb") as f:
            chunk_location, chunk_size = self.chunk_dict[b"@IMG_SCAN_03"]
            f.seek(chunk_location)
            raw = f.read(22)
            oct_header = fds_binary.oct_header.parse(raw)
            number_pixels = (
                oct_header.width * oct_header.height * oct_header.number_slices
            )
            raw_volume = np.fromstring(f.read(number_pixels * 2), dtype=np.uint16)
            volume = np.array(raw_volume)
            volume = volume.reshape(
                oct_header.width, oct_header.height, oct_header.number_slices, order="F"
            )
            volume = np.transpose(volume, [1, 0, 2])
            chunk_loc, chunk_size = self.chunk_dict.get(b"@PARAM_SCAN_04", (None, None))
            pixel_spacing = None
            if chunk_loc:
                f.seek(chunk_loc)
                scan_params = fds_binary.param_scan_04_header.parse(f.read(chunk_size))
                # NOTE: this will need reordering for dicom pixel spacing and
                # image orientation/position patient as well as possibly for nifti
                # depending on what x,y,z means here.

                # In either nifti/dicom coordinate systems, the x-y plan in raw space
                # corresponds to the x-z plane, just depends which direction.
                pixel_spacing = [
                    scan_params.x_dimension_mm / oct_header.height,  # Left/Right
                    scan_params.z_resolution_um / 1000,  # Up/Down
                    scan_params.y_dimension_mm / oct_header.width,  # Depth
                ]

        oct_volume = OCTVolumeWithMetaData(
            [volume[:, :, i] for i in range(volume.shape[2])],
            pixel_spacing=pixel_spacing,
        )
        return oct_volume

    def read_fundus_image(self):
        """Reads fundus image.

        Returns:
            obj:FundusImageWithMetaData
        """
        if b"@IMG_OBS" not in self.chunk_dict:
            raise ValueError("Could not find fundus header @IMG_OBS in chunk list")
        with open(self.filepath, "rb") as f:
            chunk_location, chunk_size = self.chunk_dict[b"@IMG_OBS"]
            f.seek(chunk_location)
            raw = f.read(21)
            fundus_header = fds_binary.fundus_header.parse(raw)
            # number_pixels = fundus_header.width * fundus_header.height * fundus_header.number_slices
            raw_image = np.fromstring(f.read(fundus_header.size), dtype=np.uint8)
            # raw_image = [struct.unpack('B', f.read(1)) for pixel in range(fundus_header.size)]
            image = np.array(raw_image)
            image = image.reshape(
                3, fundus_header.width, fundus_header.height, order="F"
            )
            image = np.transpose(image, [2, 1, 0])
            image = image.astype(np.float32)
            # store with RGB channel order
            image = np.flip(image, 2)
        fundus_image = FundusImageWithMetaData(image)
        return fundus_image

    def read_all_metadata(self, verbose=False):
        """
        Reads all available metadata and returns a dictionary.

        Args:
            verbose: If True, prints the chunks that are not supported.

        Returns:
            dict: dictionary with all metadata.
        """
        metadata = dict()
        for key in self.chunk_dict.keys():
            if key in [b"IMG_SCAN_03", b"@IMG_OBS"]:
                # these chunks have their own dedicated methods for extraction
                continue
            json_key = key.decode().split("@")[-1].lower()
            try:
                metadata[json_key] = self.read_any_info_and_make_dict(key)
            except KeyError:
                if verbose:
                    print(f"{key} there is no method for getting info from this chunk.")
        return metadata

    def read_any_info_and_make_dict(self, chunk_name):
        """
        Reads chunks, get data and make dictionary
        :param chunk_name: name of the chunk which data will be taken.
        Returns:
            dict:Chunk info Data
        """
        if chunk_name not in self.chunk_dict:
            print(f"{chunk_name} is not in chunk list, skipping.")
            return None
        with open(self.filepath, "rb") as f:
            chunk_location, chunk_size = self.chunk_dict[chunk_name]
            f.seek(chunk_location)  # Set the chunk’s current position.
            raw = f.read()
            header_name = f"{chunk_name.decode().split('@')[-1].lower()}_header"
            chunk_info_header = dict(fds_binary.__dict__[header_name].parse(raw))
            chunks_info = dict()
            for idx, key in enumerate(chunk_info_header.keys()):
                if idx == 0:
                    continue
                if type(chunk_info_header[key]) is ListContainer:
                    chunks_info[key] = list(chunk_info_header[key])
                else:
                    chunks_info[key] = chunk_info_header[key]
        return chunks_info



import os

import cv2
import numpy as np

VIDEO_TYPES = [
    ".avi",
    ".mp4",
]
IMAGE_TYPES = [".png", ".bmp", ".tiff", ".jpg", ".jpeg"]


class FundusImageWithMetaData(object):
    """Class to hold the fundus image and any related metadata, and enable saving.

    Attributes:
        image (np.array): Fundus image.
        laterality (str): Left or right eye.
        patient_id (str): Patient ID.
        image_id (str): Image ID.
        DOB (str): Patient date of birth.
    """

    def __init__(
        self, image, laterality=None, patient_id=None, image_id=None, patient_dob=None
    ):
        self.image = image
        self.laterality = laterality
        self.patient_id = patient_id
        self.image_id = image_id
        self.DOB = patient_dob

    def save(self, filepath):
        """Saves fundus image.

        Args:
            filepath (str): Location to save volume to. Extension must be in IMAGE_TYPES.
        """
        extension = os.path.splitext(filepath)[1]
        if extension.lower() in IMAGE_TYPES:
            # change channel order from RGB to BGR and save with cv2
            image = cv2.cvtColor(self.image, cv2.COLOR_RGB2BGR)
            cv2.imwrite(filepath, image)
        elif extension.lower() == ".npy":
            np.save(filepath, self.image)
        else:
            raise NotImplementedError(
                "Saving with file extension {} not supported".format(extension)
            )



import io
import struct
from pathlib import Path

import numpy as np
from construct import ListContainer
from PIL import Image

from oct_converter.image_types import FundusImageWithMetaData, OCTVolumeWithMetaData
from oct_converter.readers.binary_structs import fda_binary


class FDA(object):
    """Class for extracting data from Topcon's .fda file format."""

    def __init__(self, filepath, printing=True):
        self.filepath = Path(filepath)
        if not self.filepath.exists():
            raise FileNotFoundError(self.filepath)

        self.chunk_dict = self.get_list_of_file_chunks(printing=printing)

    def get_list_of_file_chunks(self, printing=True):
        """Find all data chunks present in the file.

        Returns:
            dict
        """
        chunk_dict = {}
        with open(self.filepath, "rb") as f:
            # skip header
            raw = f.read(15)
            header = fda_binary.header.parse(raw)

            eof = False
            while not eof:
                chunk_name_size = np.fromstring(f.read(1), dtype=np.uint8)[0]
                if chunk_name_size == 0:
                    eof = True
                else:
                    chunk_name = f.read(chunk_name_size)
                    chunk_size = np.fromstring(f.read(4), dtype=np.uint32)[0]
                    chunk_location = f.tell()
                    f.seek(chunk_size, 1)
                    if chunk_name in chunk_dict.keys():
                        previous_location = chunk_dict[chunk_name][0]
                        previous_size = chunk_dict[chunk_name][1]
                        if not isinstance(previous_location, list):
                            previous_location = [previous_location]
                            previous_size = [previous_size]
                        chunk_location = previous_location + [chunk_location]
                        chunk_size = previous_size + [chunk_size]

                    chunk_dict[chunk_name] = [chunk_location, chunk_size]
        if printing:
            print("File {} contains the following chunks:".format(self.filepath))
            for key in chunk_dict.keys():
                print(key)
            print("")
        return chunk_dict

    def read_oct_volume(self):
        """Reads OCT data.

        Notes:
            Mostly based on description of .fda file format here:
            https://bitbucket.org/uocte/uocte/wiki/Topcon%20File%20Format

        Returns:
            obj:OCTVolumeWithMetaData
        """

        if b"@IMG_JPEG" not in self.chunk_dict:
            print("@IMG_JPEG is not in chunk list, skipping.")
            return None
        with open(self.filepath, "rb") as f:
            chunk_location, chunk_size = self.chunk_dict[b"@IMG_JPEG"]
            f.seek(chunk_location)  # Set the chunk’s current position.
            raw = f.read(25)
            oct_header = fda_binary.oct_header.parse(raw)

            volume = []
            for i in range(oct_header.number_slices):
                size = np.fromstring(f.read(4), dtype=np.int32)[0]
                raw_slice = f.read(size)
                image = Image.open(io.BytesIO(raw_slice))
                volume.append(np.asarray(image))

        # read segmentation contours if possible and store them as distance
        # from top of scan to be compatible with plotting in OCTVolume
        contours = self.read_segmentation()
        if contours:
            contours = {k: oct_header.height - v for k, v in contours.items()}

        # read all other metadata
        metadata = self.read_all_metadata()

        oct_volume = OCTVolumeWithMetaData(volume, contours=contours, metadata=metadata)
        return oct_volume

    def read_oct_volume_2(self):
        """Reads OCT data. Worth trying if read_oct_volume fails.

        Returns:
            obj:OCTVolumeWithMetaData
        """

        if b"@IMG_MOT_COMP_03" not in self.chunk_dict:
            print("@IMG_MOT_COMP_03 is not in chunk list, skipping.")
            return None
        with open(self.filepath, "rb") as f:
            chunk_location, chunk_size = self.chunk_dict[b"@IMG_MOT_COMP_03"]
            f.seek(chunk_location)  # Set the chunk’s current position.
            raw = f.read(22)
            oct_header = fda_binary.oct_header_2.parse(raw)
            number_pixels = (
                oct_header.width * oct_header.height * oct_header.number_slices
            )
            raw_volume = np.fromstring(f.read(number_pixels * 2), dtype=np.uint16)
            volume = np.array(raw_volume)
            volume = volume.reshape(
                oct_header.width, oct_header.height, oct_header.number_slices, order="F"
            )
            volume = np.transpose(volume, [1, 0, 2])
        oct_volume = OCTVolumeWithMetaData(
            [volume[:, :, i] for i in range(volume.shape[2])]
        )
        return oct_volume

    def read_fundus_image(self):
        """Reads fundus image.

        Returns:
            obj:FundusImageWithMetaData
        """
        if b"@IMG_FUNDUS" not in self.chunk_dict:
            print("@IMG_FUNDUS is not in chunk list, skipping.")
            return None
        with open(self.filepath, "rb") as f:
            chunk_location, chunk_size = self.chunk_dict[b"@IMG_FUNDUS"]
            f.seek(chunk_location)  # Set the chunk’s current position.
            raw = f.read(24)  # skip 24 is important
            fundus_header = fda_binary.fundus_header.parse(raw)
            number_pixels = fundus_header.width * fundus_header.height * 3
            raw_image = f.read(fundus_header.size)
            image = Image.open(io.BytesIO(raw_image))
            image = np.asarray(image)
            # store with RGB channel order
            image = np.flip(image, 2)
        fundus_image = FundusImageWithMetaData(image)
        return fundus_image

    def read_fundus_image_gray_scale(self):
        """Reads gray scale fundus image.

        Returns:
            obj:FundusImageWithMetaData
        """
        if b"@IMG_TRC_02" not in self.chunk_dict:
            print("@IMG_TRC_02 is not in chunk list, skipping.")
            return None
        with open(self.filepath, "rb") as f:
            chunk_location, chunk_size = self.chunk_dict[b"@IMG_TRC_02"]
            f.seek(chunk_location)  # Set the chunk’s current position.
            raw = f.read(21)  # skip 21 is important
            img_trc_02_header = fda_binary.img_trc_02_header.parse(raw)
            number_pixels = img_trc_02_header.width * img_trc_02_header.height * 1
            raw_image = f.read(img_trc_02_header.size)
            image = Image.open(io.BytesIO(raw_image))
            image = np.asarray(image)
        fundus_gray_scale_image = FundusImageWithMetaData(image)
        return fundus_gray_scale_image

    def read_segmentation(self):
        """
        Reads layer segmentation data.

        Segmentation values are returned in a dictionary with a key for every
        layer boundary and are measured in pixels from B-scan bottom.

        Returns
            dict: dictionary with segmentation data.
        """

        if b"@CONTOUR_INFO" not in self.chunk_dict.keys():
            print("The file does not have any segmentation chunk.")
            return None

        layer = {
            "MULTILAYERS_1": "ILM",
            "MULTILAYERS_2": "RNFL_GCL",
            "MULTILAYERS_3": "GCL_IPL",
            "MULTILAYERS_4": "IPL_INL",
            "MULTILAYERS_5": "MZ_EZ",
            "MULTILAYERS_6": "IZ_RPE",
            "MULTILAYERS_7": "BM",
            "MULTILAYERS_8": "INL_OPL",
            "MULTILAYERS_9": "ELM",
            "MULTILAYERS_10": "CSI",
        }

        seg_dict = {}
        with open(self.filepath, "rb") as f:
            for pos in self.chunk_dict[b"@CONTOUR_INFO"][0]:
                f.seek(pos)
                raw = f.read(34)
                header = dict(fda_binary.__dict__["contour_info_header"].parse(raw))
                n_voxel = header["width"] * header["height"]

                data_bytes = f.read(n_voxel * 2)
                seg = struct.unpack("H" * n_voxel, data_bytes)
                seg = np.array(seg).reshape((header["height"], header["width"]))
                seg = np.flip(seg, 0)
                layer_name = layer.get(header["id"], header["id"])
                seg_dict[layer_name] = seg

        return seg_dict

    def read_all_metadata(self, verbose=False):
        """
        Reads all available metadata and returns a dictionary.

        Args:
            verbose: If True, prints the chunks that are not supported.

        Returns:
            dict: dictionary with all metadata.
        """
        metadata = dict()
        for key in self.chunk_dict.keys():
            if key in [b"@IMG_JPEG", b"@IMG_FUNDUS", b"@IMG_TRC_02", b"@CONTOUR_INFO"]:
                # these chunks have their own dedicated methods for extraction
                continue
            json_key = key.decode().split("@")[-1].lower()
            try:
                metadata[json_key] = self.read_any_info_and_make_dict(key)
            except KeyError:
                if verbose:
                    print(f"{key} there is no method for getting info from this chunk.")
        return metadata

    def read_any_info_and_make_dict(self, chunk_name):
        """
        Reads chunks, get data and make dictionary
        :param chunk_name: name of the chunk which data will be taken.
        Returns:
            dict:Chunk info Data
        """
        if chunk_name not in self.chunk_dict:
            print(f"{chunk_name} is not in chunk list, skipping.")
            return None
        with open(self.filepath, "rb") as f:
            chunk_location, chunk_size = self.chunk_dict[chunk_name]
            f.seek(chunk_location)  # Set the chunk’s current position.
            raw = f.read()
            header_name = f"{chunk_name.decode().split('@')[-1].lower()}_header"
            chunk_info_header = dict(fda_binary.__dict__[header_name].parse(raw))
            chunks_info = dict()
            for idx, key in enumerate(chunk_info_header.keys()):
                if idx == 0:
                    continue
                if type(chunk_info_header[key]) is ListContainer:
                    chunks_info[key] = list(chunk_info_header[key])
                else:
                    chunks_info[key] = chunk_info_header[key]
        return chunks_info



from pathlib import Path

import numpy as np

from oct_converter.image_types import OCTVolumeWithMetaData


class IMG(object):
    """Class for extracting data from Zeiss's .img file format.

    Attributes:
        filepath (str): Path to .img file for reading.
    """

    def __init__(self, filepath):
        self.filepath = Path(filepath)
        if not self.filepath.exists():
            raise FileNotFoundError(self.filepath)

    def read_oct_volume(self, rows=1024, cols=512, interlaced=False):
        """Reads OCT data.
        Args:
            rows (int): Can be used to specify a custom row dimension of the image slice. Defaults to 1024 pixels if not specified.
            cols (int): Can be used to specify a custom column dimension of the image slice. Defaults to 512 pixels if not specified.
            interlaced (bool): Determines whether data needs to be de-interlaced.

            Returns:
                obj:OCTVolumeWithMetaData
        """
        with open(self.filepath, "rb") as f:
            volume = np.frombuffer(
                f.read(), dtype=np.uint8
            )  # np.fromstring() gives numpy depreciation warning
            num_slices = len(volume) // (rows * cols)
            volume = volume.reshape((rows, cols, num_slices), order="F")
            if interlaced:
                shape = volume.shape
                interlaced = np.zeros((int(shape[0] / 2), shape[1], shape[2] * 2))
                interlaced[..., 0::2] = volume[:cols, ...]
                interlaced[..., 1::2] = volume[cols:, ...]
                interlaced = np.rot90(interlaced, axes=(0, 1))
                volume = interlaced

        oct_volume = OCTVolumeWithMetaData(
            [volume[:, :, i] for i in range(volume.shape[2])]
        )
        return oct_volume



from construct import Float32n, Float64n, Int8un, Int16un, Int32un, PaddedString, Struct

"""
        Notes:
        Mostly based on description of .fda file format here:
        https://bitbucket.org/uocte/uocte/wiki/Topcon%20File%20Format

        header (obj:Struct): Defines structure of volume's header.
        oct_header (obj:Struct): Defines structure of OCT header.
        fundus_header (obj:Struct): Defines structure of fundus header.
        chunk_dict (dict): Name of data chunks present in the file, and their start locations.
        hw_info_03_header (obj:Struct) : Defines structure of hw info header
        patient_info_02_header (obj:Struct) : Defines patient info header
        file_info_header (obj:Struct) : Defines fda file info header
        capture_info_02_header (obj:Struct) : Defines capture info header
        param_scan_04_header (obj:Struct) : Defines param scan header
        img_trc_02_header (obj:Struct) : Defines img trc header
        param_obs_02_header (obj:Struct) : Defines param obs header
        img_mot_comp_03_header (obj:Struct) : Defines img mot comp header
        effective_scan_range_header (obj:Struct) : Defines effective scan range header
        regist_info_header (obj:Struct) : Defines regist info header
        result_cornea_curve_header (obj:Struct) : Defines result cornea curve header
        result_cornea_thickness_header (obj:Struct) : Defines result cornea thickness header
        contour_info_header (obj:Struct) : Defines contour info header
        align_info_header (obj:Struct) : Defines align info header
        fast_q2_info_header (obj:Struct) : Defines fast q2 info header
        gla_littmann_01_header (obj : Struct) : Defines gla littmann 01 header
"""

header = Struct(
    "FOCT" / PaddedString(4, "ascii"),
    "FDA" / PaddedString(3, "ascii"),
    "version_info_1" / Int32un,
    "version_info_2" / Int32un,
)

oct_header = Struct(
    "type" / PaddedString(1, "ascii"),
    "unknown1" / Int32un,
    "unknown2" / Int32un,
    "width" / Int32un,
    "height" / Int32un,
    "number_slices" / Int32un,
    "unknown3" / Int32un,
)

oct_header_2 = Struct(
    "unknown" / PaddedString(1, "ascii"),
    "width" / Int32un,
    "height" / Int32un,
    "bits_per_pixel" / Int32un,
    "number_slices" / Int32un,
    "unknown" / PaddedString(1, "ascii"),
    "size" / Int32un,
)

fundus_header = Struct(
    "width" / Int32un,
    "height" / Int32un,
    "bits_per_pixel" / Int32un,
    "number_slices" / Int32un,
    "unknown" / PaddedString(4, "ascii"),
    "size" / Int32un,
    # 'img' / Int8un,
)

hw_info_03_header = Struct(
    "model_name" / PaddedString(16, "ascii"),
    "serial_number" / PaddedString(16, "ascii"),
    "zeros" / PaddedString(32, "ascii"),
    "version" / PaddedString(16, "ascii"),
    "build_year" / Int16un,
    "build_month" / Int16un,
    "build_day" / Int16un,
    "build_hour" / Int16un,
    "build_minute" / Int16un,
    "build_second" / Int16un,
    "zeros" / PaddedString(8, "ascii"),
    "version_numbers" / PaddedString(8, "ascii"),
)

patient_info_02_header = Struct(
    "patient_id" / PaddedString(8, "ascii"),
    "patient_given_name" / PaddedString(8, "ascii"),
    "patient_surname" / PaddedString(8, "ascii"),
    "birth_date_type" / Int8un,
    "birth_year" / Int16un,
    "birth_month" / Int16un,
    "birth_day" / Int16un,
)

file_info_header = Struct(
    "0x2" / Int32un,
    "0x3e8" / Int32un,
    "8.0.1.20198" / PaddedString(32, "ascii"),
)

capture_info_02_header = Struct(
    "x" / Int16un,
    "zeros" / PaddedString(52, "ascii"),
    "aquisition_year" / Int16un,
    "aquisition_month" / Int16un,
    "aquisition_day" / Int16un,
    "aquisition_hour" / Int16un,
    "aquisition_minute" / Int16un,
    "aquisition_second" / Int16un,
)

param_scan_04_header = Struct(
    "unknown" / Int16un[6],
    "x_dimension_mm" / Float64n,
    "y_dimension_mm" / Float64n,
    "z_resolution_um" / Float64n,
)

img_trc_02_header = Struct(
    "width" / Int32un,
    "height" / Int32un,
    "bits_per_pixel" / Int32un,
    "num_slices_0x2" / Int32un,
    "0x1" / Int8un,
    "size" / Int32un,
)

param_obs_02_header = Struct(
    "camera_model" / PaddedString(12, "utf16"),
    "jpeg_quality" / Int8un,
    "color_temparature" / Int8un,
)

img_mot_comp_03_header = Struct(
    "0x0" / Int8un,
    "width" / Int32un,
    "height" / Int32un,
    "bits_per_pixel" / Int32un,
    "num_slices" / Int32un,
)

img_projection_header = Struct(
    "width" / Int32un,
    "height" / Int32un,
    "bits_per_pixel" / Int32un,
    "0x1000002" / Int32un,
    "size" / Int32un,
)

effective_scan_range_header = Struct(
    "bounding_box_fundus_pixel" / Int32un[4],
    "bounding_box_trc_pixel" / Int32un[4],
)

regist_info_header = Struct(
    "0x0" / Int8un,
    "unknown" / Int32un[2],
    "bounding_box_in_fundus_pixels" / Int32un[4],
    "version" / PaddedString(32, "ascii"),
    "bounding_box_in_trc_pixels" / Int32un[4],
)

result_cornea_curve_header = Struct(
    "id" / Int8un[20],
    "width" / Int32un,
    "height" / Int32un,
    "8.0.1.21781" / Int8un[32],
)

result_cornea_thickness_header = Struct(
    "8.0.1.21781" / Int8un[32],
    "id" / Int8un[20],
    "width" / Int32un,
    "height" / Int32un,
)

contour_info_header = Struct(
    "id" / PaddedString(20, "ascii"),
    "type" / Int16un,
    "width" / Int32un,
    "height" / Int32un,
    "size" / Int32un,
)

fast_q2_info_header = Struct("various_quality_statistics" / Float32n[6])

gla_littmann_01_header = Struct("0xffff" / Int32un, "0x1" / Int32un)



from oct_converter.readers import BOCT

filepath = "../sample_files/sample.OCT"
boct = BOCT(filepath)
oct_volumes = boct.read_oct_volume(
    diskbuffered=True
)  # returns an OCT volume with additional metadata if available
for oct in oct_volumes:
    oct.peek()  # plots a montage of the volume
    oct.save("boct_testing.avi")  # save volume as a movie
    oct.save(
        "boct_testing.png"
    )  # save volume as a set of sequential images, fds_testing_[1...N].png
